-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

local LOCATOR_LAYOUT_NAME = "locator";

local worldMapShowPosition = setmetatable({},
{
  __index = function(tbl, key)
    -- Local Variables
    local mapId = WorldMapFrame:GetMapID();
    local mapInfo;

    -- Set an initial value for the specified key
    tbl[key] = false;

    -- Check the world map frame is displaying a map
    if (mapId ~= nil) then
      -- Get the map's info
      mapInfo = C_Map.GetMapInfo(mapId);
      if (mapInfo ~= nil) then
        tbl[key] = ((mapInfo.mapType ~= Enum.UIMapType.Cosmic   ) and
                    (mapInfo.mapType ~= Enum.UIMapType.World    ) and
                    (mapInfo.mapType ~= Enum.UIMapType.Continent));
      end
    end

    -- Save the value in the table
    return tbl[key];
  end
});

local locator = nil;
local mapLocation = nil;

local mapInfo =
{
  maps     = {},
  position = CreateVector2D(0, 0),
  vectors  = {CreateVector2D(0, 0), CreateVector2D(1, 1)},
}

-------------------------------------------------------------------------------
--             L  O  C  A  L     D  E  F  I  N  I  T  I  O  N  S
-------------------------------------------------------------------------------

local locatorUpdate;
 
-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to process events for the Locator frame
--
local function locatorOnClick(self, button)
  -- Local Variables
  local activeWindow = ChatEdit_GetActiveWindow();
  local currentText;
  local cursorPos;
  local numLetters;
  local posX;
  local posY;
  local zoneText = nil;

  -- Check there is an active window
  if (activeWindow ~= nil) then
    -- Get info from the active window
    currentText = activeWindow:GetText();
    cursorPos   = activeWindow:GetCursorPosition();
    numLetters  = activeWindow:GetNumLetters();

    -- Get the name of the (sub-)zone
    if (IsShiftKeyDown()) then
      zoneText = GetSubZoneText();
    end
    if ((zoneText == nil) or (zoneText == "")) then
      zoneText = GetRealZoneText();
    end

    -- Add the (sub-)zone and location to the active chat window
    if (zoneText ~= nil) then
      posX, posY = L.locationGetPlayer();
      activeWindow:Insert(format("%s%s (%2.1f, %2.1f)%s",
        (((cursorPos > 0) and
          (currentText:sub(cursorPos, cursorPos) ~= " ")) and " " or ""),
        zoneText, posX, posY,
        (((cursorPos < numLetters) and
          (currentText:sub(cursorPos+1, cursorPos+1) ~= " ")) and " " or "")));
    end
  end

  return;
end

--
-- Function to display the tooltip for the Locator frame
--
local function locatorOnEnter(self)
  -- Show a tooltip for the Locator frame
  GameTooltip_SetDefaultAnchor(GameTooltip, UIParent);
  GameTooltip:SetText(L["Locator"], HIGHLIGHT_FONT_COLOR.r,
    HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b, nil, true);
  GameTooltip:AddLine(L["LocatorTooltip"]);
  GameTooltip:SetMinimumWidth(135, true);
  GameTooltip:Show();

  return;
end

--
-- Function to process events for the Locator frame
--
local function locatorOnEvent(self, event, ...)
  -- Process the event
  if (event == "PET_BATTLE_OPENING_START") then
    self:Hide();
  elseif (event == "PET_BATTLE_CLOSE") then
    self:SetShown(NigglesDBC.settings.LocatorShown);
  end

  return;
end

--
-- Function called when the user stops dragging the Locator frame
--
local function locatorOnDragStop(self)
  -- Stop moving the Locator frame and make sure its position is valid
  self:StopMovingOrSizing();
  ValidateFramePosition(self);

  -- Save the Locator frame's position
  L.layoutSave(LOCATOR_LAYOUT_NAME, self, false);

  return;
end

--
-- Function to hide the tooltip for the Locator frame
--
local function locatorOnLeave(self)
  -- Hide the tooltip
  GameTooltip:SetMinimumWidth(0, false);
  GameTooltip:Hide();

  return;
end

--
-- Function called when the Locator frame is shown
--
local function locatorOnShow(self)
  -- Restore the locator's position
  L.layoutRestore(LOCATOR_LAYOUT_NAME, self, false);

  -- Start updating the locator
  locatorUpdate();

  return
end

--
-- Function to update the coordinates displayed in the Locator frame
--
locatorUpdate = function()
  -- Local Variables
  local posX;
  local posY;

  -- Check if the Locator is visible
  if (locator:IsShown()) then
    -- Get the player's position
    posX, posY = L.locationGetPlayer();
    if ((posX ~= nil) and (posY ~= nil)) then
      -- Update the Locator's X position, if required
      if (posX ~= locator.lastX) then
        locator.posX:SetFormattedText("%2.1f,", posX);
        locator.lastX = posX;
      end

      -- Update the Locator's Y position, if required
      if (posY ~= locator.lastY) then
        locator.posY:SetFormattedText("%2.1f", posY);
        locator.lastY = posY;
      end
    else
      if ((locator.lastX ~= -1) or (locator.lastY ~= -1)) then
        locator.posX:SetText("");
        locator.posY:SetText("");
        locator.lastX = -1;
        locator.lastY = -1;
      end
    end
  
    -- Update the locator again after a short while
    C_Timer.After(0.1, locatorUpdate);
  end

  return;
end

--
-- Function to update the coordinates displayed in the Map Locator frame
--
local function mapLocationOnUpdate(self, elapsed)
  -- Local Variables
   local cursorX = -1;
   local cursorY = -1;
   local mapFrame = WorldMapFrame.ScrollContainer.Child;
   local mapId;

  -- Check the map location should be shown
  if (NigglesDBC.settings.UiMapLocation == true) then
    -- Check the mouse is over the map
    if (mapFrame:IsMouseOver()) then
      mapId = WorldMapFrame:GetMapID();
      if ((mapId ~= nil) and (worldMapShowPosition[mapId])) then
        -- Get the position of the mouse
        local mapCentreX, mapCentreY = mapFrame:GetCenter();
        local mapWidth  = mapFrame:GetWidth();
        local mapHeight = mapFrame:GetHeight();
        local mapScale  = mapFrame:GetEffectiveScale();

        -- Get and convert the cursor's position into a map position
        cursorX, cursorY = GetCursorPosition();
        cursorX = ((cursorX/mapScale)-(mapCentreX-(mapWidth/2)))/mapWidth;
        cursorY = (mapCentreY+(mapHeight/2)-(cursorY/mapScale))/mapHeight;
      end
    end

    -- Check if the cursor position should be displayed
    if ((cursorX >= 0) and (cursorY >= 0)) then
      -- Change the cursor position to be 1 decimal place
      cursorX = math.floor(cursorX*1000)/10;
      cursorY = math.floor(cursorY*1000)/10;

      -- Update the Map Locator's X position, if required
      if (cursorX ~= self.lastX) then
        self.lastX = cursorX;
        self.posX:SetFormattedText("%2.01f,", cursorX);
      end

      -- Update the Map Locator's Y position, if required
      if (cursorY ~= self.lastY) then
        self.lastY = cursorY;
        self.posY:SetFormattedText("%2.01f", cursorY);
      end
    elseif ((self.lastX ~= -1) or (self.lastY ~= -1)) then
      -- Clear the Map Locator's position
      self.posX:SetText("");
      self.posY:SetText("");
      self.lastX = -1;
      self.lastY = -1;
    end
  end

  return;
end

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to get the player's location
--
-- NOTE: Based on code posted to a WoWInterface forum by Elcius on 2018-06-15
--
function L.locationGetPlayer()
  -- Local Variables
  local _;
  local mapId = C_Map.GetBestMapForUnit("player");
  local mapVectors = mapInfo.maps[mapId];
  local position = mapInfo.position;

  -- Check the map ID could be obtained
  if (mapId ~= nil) then
    -- Check if the map's vectors need to be calculated
    if (mapVectors == nil) then
      -- Get the vectors of the NE and SW corners of the map
      mapVectors =
      {
        select(2, C_Map.GetWorldPosFromMapPos(mapId, mapInfo.vectors[1])),
        select(2, C_Map.GetWorldPosFromMapPos(mapId, mapInfo.vectors[2])),
      };

      -- Convert the NE vector into a multiplier for the map
      mapVectors[2]:Subtract(mapVectors[1]);
      mapVectors[2].x = 1/mapVectors[2].x;
      mapVectors[2].y = 1/mapVectors[2].y;

      -- Save the map's vectors
      mapInfo.maps[mapId] = mapVectors;
    end

    -- Get the player's world position
    position.x, position.y = UnitPosition("player");

    -- Return the player's world position as a map position
    if (position.x ~= nil) and (position.y ~= nil) then
      position:Subtract(mapVectors[1]);
      position.x = math.floor(mapVectors[2].x*position.x*1000)/10;
      position.y = math.floor(mapVectors[2].y*position.y*1000)/10;
    end
  else
    position.x = nil;
    position.y = nil;
  end

  -- Return the player's position
  -- NOTE: The X and Y coordinates are reversed deliberately. It appears to
  --       be required as a by-product of the vector arithmetic.
  return position.y, position.x;
end

--
-- Function to enable/disable the Map Location frame
--
function L.mapLocationEnable()
  -- Check if the Map Location frame needs to be created
  if ((mapLocation == nil) and
      (NigglesDBC.settings.UiMapLocation == true)) then
    -- Create the Map Location frame
    mapLocation = CreateFrame("FRAME", nil, WorldMapFrame.BorderFrame,
       "NigglesMapLocationTemplate");
    mapLocation:SetPoint("TOPRIGHT", 
      WorldMapFrame.BorderFrame.MaximizeMinimizeFrame, "TOPLEFT", 1, -6);
    mapLocation:SetFrameLevel(
      WorldMapFrame.BorderFrame.NineSlice:GetFrameLevel()+50);
    mapLocation.lastX = 0;
    mapLocation.lastY = 0;

    -- Set the frame's script handlers
    mapLocation:SetScript("OnUpdate", mapLocationOnUpdate);
  end

  -- Show/Hide the Map Location frame, if required
  if (mapLocation ~= nil) then
    mapLocation:SetShown(NigglesDBC.settings.UiMapLocation);
  end

  return;
end

-------------------------------------------------------------------------------
--               G  L  O  B  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to toggle the displaying of the Locator frames
--
function NigglesLocatorShow(show)
  -- Local Variables
  local frame;

  -- Work out the correct state for the Locator frames
  if (show ~= nil) then
    NigglesDBC.settings.LocatorShown = show;
  else
    NigglesDBC.settings.LocatorShown = not NigglesDBC.settings.LocatorShown;
  end

  -- Check if the locator frame needs to be created
  if ((NigglesDBC.settings.LocatorShown) and (locator == nil)) then
    -- Create the locator frame
    locator = CreateFrame("BUTTON", nil, UIParent,
       "NigglesLocatorTemplate");

    -- Register the frame for drag operations
    locator:RegisterForDrag("LeftButton");

    -- Set the frame's script handlers
    locator:SetScript("OnClick",    locatorOnClick);
    locator:SetScript("OnDragStop", locatorOnDragStop);
    locator:SetScript("OnEnter",    locatorOnEnter);
    locator:SetScript("OnEvent",    locatorOnEvent);
    locator:SetScript("OnLeave",    locatorOnLeave);
    locator:SetScript("OnShow",     locatorOnShow);

    -- Register for some events
    locator:RegisterEvent("PET_BATTLE_OPENING_START");
    locator:RegisterEvent("PET_BATTLE_CLOSE");

    -- Initialise the last position displayed
    locator.lastX = 0;
    locator.lastY = 0;
  end

    -- Initialise the last position displayed
  -- Show/Hide the Locator frame, if required
  if ((locator ~= nil) and (not C_PetBattles.IsInBattle())) then
    locator:SetShown(NigglesDBC.settings.LocatorShown);
  end

  return;
end
